@@ -6,11 +6,15 @@ My own variation on function-specific inspect-like features.
 # Copyright (c) 2009 Gael Varoquaux
 # License: BSD Style, 3 clauses.
 
-import itertools
+from itertools import islice
 import inspect
 import warnings
+import re
 import os
 
+from ._compat import _basestring
+from .logger import pformat
+
 
 def get_func_code(func):
     """ Attempts to retrieve a reliable function code hash.
@@ -35,23 +39,36 @@ def get_func_code(func):
     """
     source_file = None
     try:
+        code = func.__code__
+        source_file = code.co_filename
+        if not os.path.exists(source_file):
+            # Use inspect for lambda functions and functions defined in an
+            # interactive shell, or in doctests
+            source_code = ''.join(inspect.getsourcelines(func)[0])
+            line_no = 1
+            if source_file.startswith('<doctest '):
+                source_file, line_no = re.match(
+                            '\<doctest (.*\.rst)\[(.*)\]\>',
+                            source_file).groups()
+                line_no = int(line_no)
+                source_file = '<doctest %s>' % source_file
+            return source_code, source_file, line_no
         # Try to retrieve the source code.
-        source_file = func.func_code.co_filename
-        source_file_obj = file(source_file)
-        first_line = func.func_code.co_firstlineno
-        # All the lines after the function definition:
-        source_lines = list(itertools.islice(source_file_obj, first_line - 1,
-                                             None))
+        with open(source_file) as source_file_obj:
+            first_line = code.co_firstlineno
+            # All the lines after the function definition:
+            source_lines = list(islice(source_file_obj, first_line - 1, None))
         return ''.join(inspect.getblock(source_lines)), source_file, first_line
     except:
         # If the source code fails, we use the hash. This is fragile and
         # might change from one session to another.
-        if hasattr(func, 'func_code'):
-            return str(func.func_code.__hash__()), source_file, -1
+        if hasattr(func, '__code__'):
+            # Python 3.X
+            return str(func.__code__.__hash__()), source_file, -1
         else:
-            # Weird objects like numpy ufunc don't have func_code
+            # Weird objects like numpy ufunc don't have __code__
             # This is fragile, as quite often the id of the object is
-            # in the repr, so it might not persist accross sessions,
+            # in the repr, so it might not persist across sessions,
             # however it will work for ufuncs.
             return repr(func), source_file, -1
 
@@ -59,8 +76,13 @@ def get_func_code(func):
 def _clean_win_chars(string):
     "Windows cannot encode some characters in filenames"
     import urllib
+    if hasattr(urllib, 'quote'):
+        quote = urllib.quote
+    else:
+        # In Python 3, quote is elsewhere
+        quote = urllib.parse.quote
     for char in ('<', '>', '!', ':', '\\'):
-        string = string.replace(char, urllib.quote(char))
+        string = string.replace(char, quote(char))
     return string
 
 
@@ -73,7 +95,7 @@ def get_func_name(func, resolv_alias=True, win_characters=True):
         func: callable
             The func to inspect
         resolv_alias: boolean, optional
-            If true, possible local alias are indicated.
+            If true, possible local aliases are indicated.
         win_characters: boolean, optional
             If true, substitute special characters using urllib.quote
             This is useful in Windows, as it cannot encode some filenames
@@ -87,7 +109,7 @@ def get_func_name(func, resolv_alias=True, win_characters=True):
             if hasattr(func, '__class__'):
                 module = func.__class__.__module__
             else:
-                module = 'unkown'
+                module = 'unknown'
     if module is None:
         # Happens in doctests, eg
         module = ''
@@ -98,8 +120,13 @@ def get_func_name(func, resolv_alias=True, win_characters=True):
             filename = None
         if filename is not None:
             # mangling of full path to filename
-            filename = filename.replace(os.sep, '-')
-            filename = filename.replace(":", "-")
+            parts = filename.split(os.sep)
+            if parts[-1].startswith('<ipython-input'):
+                # function is defined in an IPython session. The filename
+                # will change with every new kernel instance. This hack
+                # always returns the same filename
+                parts[-1] = '__ipython-input__'
+            filename = '-'.join(parts)
             if filename.endswith('.py'):
                 filename = filename[:-3]
             module = module + '-' + filename
@@ -128,7 +155,7 @@ def get_func_name(func, resolv_alias=True, win_characters=True):
     return module, name
 
 
-def filter_args(func, ignore_lst, *args, **kwargs):
+def filter_args(func, ignore_lst, args=(), kwargs=dict()):
     """ Filters the given args and kwargs using a list of arguments to
         ignore, and a function specification.
 
@@ -147,12 +174,10 @@ def filter_args(func, ignore_lst, *args, **kwargs):
         Returns
         -------
         filtered_args: list
-            List of filtered positional arguments.
-        filtered_kwdargs: dict
-            List of filtered Keyword arguments.
+            List of filtered positional and keyword arguments.
     """
     args = list(args)
-    if isinstance(ignore_lst, basestring):
+    if isinstance(ignore_lst, _basestring):
         # Catch a common mistake
         raise ValueError('ignore_lst must be a list of parameters to ignore '
             '%s (type %s) was given' % (ignore_lst, type(ignore_lst)))
@@ -175,7 +200,7 @@ def filter_args(func, ignore_lst, *args, **kwargs):
     if inspect.ismethod(func):
         # First argument is 'self', it has been removed by Python
         # we need to add it back:
-        args = [func.im_self, ] + args
+        args = [func.__self__, ] + args
     # XXX: Maybe I need an inspect.isbuiltin to detect C-level methods, such
     # as on ndarrays.
 
@@ -202,12 +227,12 @@ def filter_args(func, ignore_lst, *args, **kwargs):
                            name,
                            repr(args)[1:-1],
                            ', '.join('%s=%s' % (k, v)
-                                    for k, v in kwargs.iteritems())
+                                    for k, v in kwargs.items())
                            )
                         )
 
     varkwargs = dict()
-    for arg_name, arg_value in kwargs.iteritems():
+    for arg_name, arg_value in sorted(kwargs.items()):
         if arg_name in arg_dict:
             arg_dict[arg_name] = arg_value
         elif arg_keywords is not None:
@@ -237,3 +262,41 @@ def filter_args(func, ignore_lst, *args, **kwargs):
                                                    )))
     # XXX: Return a sorted list of pairs?
     return arg_dict
+
+
+def format_signature(func, *args, **kwargs):
+    # XXX: Should this use inspect.formatargvalues/formatargspec?
+    module, name = get_func_name(func)
+    module = [m for m in module if m]
+    if module:
+        module.append(name)
+        module_path = '.'.join(module)
+    else:
+        module_path = name
+    arg_str = list()
+    previous_length = 0
+    for arg in args:
+        arg = pformat(arg, indent=2)
+        if len(arg) > 1500:
+            arg = '%s...' % arg[:700]
+        if previous_length > 80:
+            arg = '\n%s' % arg
+        previous_length = len(arg)
+        arg_str.append(arg)
+    arg_str.extend(['%s=%s' % (v, pformat(i)) for v, i in kwargs.items()])
+    arg_str = ', '.join(arg_str)
+
+    signature = '%s(%s)' % (name, arg_str)
+    return module_path, signature
+
+
+def format_call(func, args, kwargs, object_name="Memory"):
+    """ Returns a nicely formatted statement displaying the function
+        call with the given arguments.
+    """
+    path, signature = format_signature(func, *args, **kwargs)
+    msg = '%s\n[%s] Calling %s...\n%s' % (80 * '_', object_name,
+                                          path, signature)
+    return msg
+    # XXX: Not using logging framework
+    #self.debug(msg)
